Index: sys/arm/arm/machdep.c
===================================================================
--- sys/arm/arm/machdep.c	(revision 236701)
+++ sys/arm/arm/machdep.c	(working copy)
@@ -664,7 +664,7 @@
  * Fake up a boot descriptor table
  */
 vm_offset_t
-fake_preload_metadata(void)
+fake_preload_metadata(struct arm_boot_params *abp __unused)
 {
 #ifdef DDB
 	vm_offset_t zstart = 0, zend = 0;
@@ -709,3 +709,20 @@
 
 	return (lastaddr);
 }
+
+/*
+ * Stub version of the boot parameter parsing routine.  We are
+ * called early in initarm, before even VM has been initialized.
+ * This routine needs to preserve any data that the boot loader
+ * has passed in before the kernel starts to grow past the end
+ * of the BSS, traditionally the place boot-loaders put this data.
+ *
+ * Since this is called so early, things that depend on the vm system
+ * being setup (including access to some SoC's serial ports), about
+ * all that can be done in this routine is to copy the arguments.
+ *
+ * This is the default boot parameter parsing routine.  Individual
+ * kernels/boards can override this weak function with one of their
+ * own.  We just fake metadata...
+ */
+__weak_reference(fake_preload_metadata, parse_boot_param);
Index: sys/arm/xscale/i8134x/crb_machdep.c
===================================================================
--- sys/arm/xscale/i8134x/crb_machdep.c	(revision 236701)
+++ sys/arm/xscale/i8134x/crb_machdep.c	(working copy)
@@ -191,8 +191,8 @@
 	vm_offset_t lastaddr;
 	uint32_t memsize, memstart;
 
+	lastaddr = parse_boot_param(abp);
 	set_cpufuncs();
-	lastaddr = fake_preload_metadata();
 	pcpu_init(pcpup, 0, sizeof(struct pcpu));
 	PCPU_SET(curthread, &thread0);
 
Index: sys/arm/xscale/i80321/ep80219_machdep.c
===================================================================
--- sys/arm/xscale/i80321/ep80219_machdep.c	(revision 236701)
+++ sys/arm/xscale/i80321/ep80219_machdep.c	(working copy)
@@ -194,8 +194,8 @@
 	vm_offset_t lastaddr;
 	uint32_t memsize, memstart;
 
+	lastaddr = parse_boot_param(abp);
 	set_cpufuncs();
-	lastaddr = fake_preload_metadata();
 	pcpu_init(pcpup, 0, sizeof(struct pcpu));
 	PCPU_SET(curthread, &thread0);
 
Index: sys/arm/xscale/i80321/iq31244_machdep.c
===================================================================
--- sys/arm/xscale/i80321/iq31244_machdep.c	(revision 236701)
+++ sys/arm/xscale/i80321/iq31244_machdep.c	(working copy)
@@ -195,8 +195,8 @@
 	vm_offset_t lastaddr;
 	uint32_t memsize, memstart;
 
+	lastaddr = parse_boot_param(abp);
 	set_cpufuncs();
-	lastaddr = fake_preload_metadata();
 	pcpu_init(pcpup, 0, sizeof(struct pcpu));
 	PCPU_SET(curthread, &thread0);
 
Index: sys/arm/xscale/pxa/pxa_machdep.c
===================================================================
--- sys/arm/xscale/pxa/pxa_machdep.c	(revision 236701)
+++ sys/arm/xscale/pxa/pxa_machdep.c	(working copy)
@@ -176,9 +176,8 @@
 	int i, j;
 	uint32_t memsize[PXA2X0_SDRAM_BANKS], memstart[PXA2X0_SDRAM_BANKS];
 
+	lastaddr = parse_boot_param(abp);
 	set_cpufuncs();
-
-	lastaddr = fake_preload_metadata();
 	pcpu_init(pcpup, 0, sizeof(struct pcpu));
 	PCPU_SET(curthread, &thread0);
 
Index: sys/arm/xscale/ixp425/avila_machdep.c
===================================================================
--- sys/arm/xscale/ixp425/avila_machdep.c	(revision 236701)
+++ sys/arm/xscale/ixp425/avila_machdep.c	(working copy)
@@ -240,8 +240,8 @@
 	vm_offset_t lastaddr;
 	uint32_t memsize;
 
+	lastaddr = parse_boot_param(abp);
 	set_cpufuncs();		/* NB: sets cputype */
-	lastaddr = fake_preload_metadata();
 	pcpu_init(pcpup, 0, sizeof(struct pcpu));
 	PCPU_SET(curthread, &thread0);
 
Index: sys/arm/mv/mv_machdep.c
===================================================================
--- sys/arm/mv/mv_machdep.c	(revision 236701)
+++ sys/arm/mv/mv_machdep.c	(working copy)
@@ -333,7 +333,7 @@
 	 */
 	mdp = (void *)((uint32_t)mdp & ~PAGE_MASK);
 
-	/* Parse metadata and fetch parameters */
+	/* Parse metadata and fetch parameters (move to common machdep.c?) */
 	if (mdp != NULL) {
 		preload_metadata = mdp;
 		kmdp = preload_search_by_type("elf kernel");
@@ -352,7 +352,7 @@
 		preload_addr_relocate = KERNVIRTADDR - KERNPHYSADDR;
 	} else {
 		/* Fall back to hardcoded metadata. */
-		lastaddr = fake_preload_metadata();
+		lastaddr = fake_preload_metadata(abp);
 	}
 
 #if defined(FDT_DTB_STATIC)
Index: sys/arm/include/machdep.h
===================================================================
--- sys/arm/include/machdep.h	(revision 236701)
+++ sys/arm/include/machdep.h	(working copy)
@@ -6,10 +6,12 @@
 
 /* misc prototypes used by the many arm machdeps */
 void arm_lock_cache_line(vm_offset_t);
-vm_offset_t fake_preload_metadata(void);
 void halt(void);
 void data_abort_handler(trapframe_t *);
 void prefetch_abort_handler(trapframe_t *);
 void undefinedinstruction_bounce(trapframe_t *);
+struct arm_boot_params;
+vm_offset_t parse_boot_param(struct arm_boot_params *abp);
+vm_offset_t fake_preload_metadata(struct arm_boot_params *abp);
 
 #endif /* !_MACHINE_MACHDEP_H_ */
Index: sys/arm/econa/econa_machdep.c
===================================================================
--- sys/arm/econa/econa_machdep.c	(revision 236701)
+++ sys/arm/econa/econa_machdep.c	(working copy)
@@ -197,8 +197,8 @@
 
 	boothowto = RB_VERBOSE;
 
+	lastaddr = parse_boot_param(abp);
 	set_cpufuncs();
-	lastaddr = fake_preload_metadata();
 	pcpu_init(pcpup, 0, sizeof(struct pcpu));
 	PCPU_SET(curthread, &thread0);
 
Index: sys/arm/s3c2xx0/s3c24x0_machdep.c
===================================================================
--- sys/arm/s3c2xx0/s3c24x0_machdep.c	(revision 236701)
+++ sys/arm/s3c2xx0/s3c24x0_machdep.c	(working copy)
@@ -246,10 +246,9 @@
 	int i;
 	uint32_t memsize;
 
+	boothowto = 0;  /* Likely not needed */
+	lastaddr = parse_boot_param(abp);
 	i = 0;
-
-	boothowto = 0;
-
 	set_cpufuncs();
 	cpufuncs.cf_sleep = s3c24x0_sleep;
 	lastaddr = fake_preload_metadata();
Index: sys/arm/sa11x0/assabet_machdep.c
===================================================================
--- sys/arm/sa11x0/assabet_machdep.c	(revision 236701)
+++ sys/arm/sa11x0/assabet_machdep.c	(working copy)
@@ -216,10 +216,10 @@
 	uint32_t memsize = 32 * 1024 * 1024;
 	sa1110_uart_vaddr = SACOM1_VBASE;
 
-	boothowto = RB_VERBOSE | RB_SINGLE;
+	boothowto = RB_VERBOSE | RB_SINGLE;     /* Default value */
+	lastaddr = parse_boot_param(abp);
 	cninit();
 	set_cpufuncs();
-	lastaddr = fake_preload_metadata();
 	physmem = memsize / PAGE_SIZE;
 	pc = &__pcpu;
 	pcpu_init(pc, 0, sizeof(struct pcpu));
Index: sys/arm/at91/at91_machdep.c
===================================================================
--- sys/arm/at91/at91_machdep.c	(revision 236701)
+++ sys/arm/at91/at91_machdep.c	(working copy)
@@ -394,8 +394,8 @@
 	uint32_t memsize;
 	vm_offset_t lastaddr;
 
+	lastaddr = parse_boot_param(abp);
 	set_cpufuncs();
-	lastaddr = fake_preload_metadata();
 	pcpu_init(pcpup, 0, sizeof(struct pcpu));
 	PCPU_SET(curthread, &thread0);
 
